// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "Kismet/BlueprintFunctionLibrary.h"
#include "Logging/LogVerbosity.h"
#include "Logging/TokenizedMessage.h"
#include "Templates/SharedPointer.h"
#include "MyMessageLogFunctionLibrary.generated.h"

UENUM()
enum EMessageSeverityType
{
	/* CriticalError = EMessageSeverity::CriticalError, */
	Error = EMessageSeverity::Error,
	PerformanceWarning = EMessageSeverity::PerformanceWarning,
	Warning = EMessageSeverity::Warning,
	Info = EMessageSeverity::Info,
};

UCLASS()
class MYMESSAGELOGRUNTIME_API UMyMessageLogFunctionLibrary : public UBlueprintFunctionLibrary
{
	GENERATED_BODY()

public:

	template <typename FmtType, typename... ArgTypes>
	static TSharedPtr<FTokenizedMessage> OutMessageLog(ELogVerbosity::Type Verbosity, const FName& LogName, const FString& File, int32 Line, const FmtType& Fmt, ArgTypes... Args)
	{
		const FString Message = FString::Printf(Fmt, Args...);
		return OutMessageLogInternal(Verbosity, LogName, File, Line, Message);
	}

	UFUNCTION(BlueprintCallable, CustomThunk, meta = (DisplayName = "Out PIE Log"))
	static void OutPIELogBP(EMessageSeverityType Severity = EMessageSeverityType::Info, FString Message = FString(TEXT("")));

	UFUNCTION(BlueprintCallable, CustomThunk, meta = (DisplayName = "Out Log"))
	static void OutLogBP(EMessageSeverityType Severity = EMessageSeverityType::Info, FName LogName = TEXT("PIE"), FString Message = FString(TEXT("")));

	UFUNCTION(BlueprintCallable, CustomThunk, meta = (DisplayName = "Out Log with Call Stack"))
	static void OutLogCallStackBP(EMessageSeverityType Severity = EMessageSeverityType::Info, FName LogName = TEXT("PIE"), FString Message = FString(TEXT("")));

	UFUNCTION(BlueprintCallable)
	static void RegisterLogListing(FName LogName, FText LogDisplayName);

protected:

	static TSharedPtr<FTokenizedMessage> OutMessageLogInternal(ELogVerbosity::Type Verbosity, const FName& LogName, const FString& File, int32 Line, const FString& Message);

	static TSharedPtr<FTokenizedMessage> OutMessageLogBP(EMessageSeverity::Type Severity, const FName& LogName, const UObject* ActiveObject, const FFrame& StackFrame, const FString& Message);

	DECLARE_FUNCTION(execOutPIELogBP);
	DECLARE_FUNCTION(execOutLogBP);
	DECLARE_FUNCTION(execOutLogCallStackBP);

};
